#!/usr/bin/perl
# $Id: snmp_report.pl,v 1.2 2002/02/21 16:09:13 gadsden Exp $
#
# Create formatted html report from output of snmp_get_sysDescr.pl,
# using subnets.txt file to create table of contents.
#
# Usage: snmp_report.pl [ -t tocfile ]
#
#  where tocfile := name of file containing list of subnets to be used
#                   for creating the table of contents for the report,
#                   default /usr/local/share/subnets.txt
#
# Author: Richard Gadsden <gadsden@musc.edu>
#         Medical University of South Carolina
#         Charleston, SC, USA

use Getopt::Std;

$CSSFILE = "../snmp-audit.css"; # Edit this if desired

getopts ('t:');
$tocfile = defined($opt_t) ? $opt_t : "/usr/local/share/subnets.txt";

open (TOC, "<$tocfile") or die "$0: Cannot open $tocfile: $!";
@subnets = <TOC>;
chop @subnets;
close (TOC);

&print_report_header;
foreach $subnet (@subnets) {
  &print_toc_row ($subnet);
}

$netno = 0;
$subnet = $subnets[$netno];
$subnetTop = &highest_addr ($subnet);
&print_section_header ($subnet, $subnetTop);

while (<>) {
  ($addr, $name, $result) = split (/\t/);
  until (&within_range ($addr, $subnetTop)) {
      &print_section_footer;
      $netno++;
      $subnet = $subnets[$netno];
      $subnetTop = &highest_addr ($subnet);
      &print_section_header ($subnet, $subnetTop);
  }
  &print_row ($addr, $name, $result);
}
&print_section_footer;
&print_report_footer;

sub highest_addr {
  local ($subnet) = @_;
  local ($addr, $netbits) = split (/\//, $subnet);
  local $hostbits = 32 - $netbits;
  local $netsize = 2**$hostbits;
  local @octet = split (/\./, $addr);
  $octet[3] += $netsize - 1;
  for (local $j = 3 ; $j > 0 ; $j--) {
    if ($octet[$j] >= 256) {
      $octet[$j-1] += int ($octet[$j]/256);
      $octet[$j] %= 256;
    }
  }
  return join (".", @octet);
}

sub within_range {
  local ($addr, $subnetTop) = @_;
  local @octetA = split (/\./, $addr);
  local @octetS = split (/\./, $subnetTop);
  for (local $j = 0; $j <= 3 ; $j++) {
    if ($octetA[$j] > $octetS[$j]) { return (0); }
  }
  return (1);
}

sub print_report_header {
  print "<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">\n";
  print "<html>\n";
  print "<head>\n";
  print "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n";
  print "<title>SNMP Status Report</title>\n";
  print "<link rel=\"stylesheet\" type=\"text/css\" href=\"$CSSFILE\">\n";
  print "</head>";
  print "<body>\n";
  print "<h3>Table of Contents:</h3>\n";
}

sub print_toc_row {
  local ($subnet) = @_;
  print "<a href=\"#$subnet\">$subnet</a><br>\n";
}

sub print_section_header {
  local ($subnet, $subnetTop) = @_;
  local ($addr, $netbits) = split (/\//, $subnet);
  print "<a name=\"$subnet\"><h3>$subnet ($addr - $subnetTop)</h3></a>\n";
  print "<p><table border=1 cellpadding=3 width=90%>\n";
  print "<colgroup><col width=15%><col width=15%><col width=70%></colgroup>\n";
  print "<tr><th align=left>Address</th>";
  print "<th align=left>DNS Name</th>";
  print "<th align=left>system.sysDescr.0</th></tr>\n";
  return;
}

sub print_row {
  local ($addr, $name, $result) = @_;
  print "<tr><td>$addr&nbsp;</td><td>$name&nbsp;</td><td>$result&nbsp;</td></tr>\n";
  return;
}

sub print_section_footer {
  print "</table></p>\n";
  return;
}

sub print_report_footer {
  print "</body>\n";
  print "</html>\n";
}
